"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var player_1 = require("../src/player");
var pile_1 = require("../src/pile");
var market_1 = require("../src/market");
var moves_1 = require("../src/moves");
var shapes_1 = require("../src/shapes");
var card_1 = require("../src/card");
var events_1 = require("../src/events");
var emitter = new events_1.EventEmitter();
var pile = new pile_1.default({ emitter: emitter });
pile.push([(0, card_1.GetCircle)({ value: 3 })]);
var market = new market_1.default({ noOfDecks: 1, pile: function () { return pile; }, emitter: emitter });
var mockMarket = function () { return market; };
var mockPile = function () { return pile; };
describe("Player", function () {
    describe("Constructor", function () {
        it("should throw InvalidArgumentError if props argument is not supplied", function () {
            try {
                // @ts-ignore
                var player = new player_1.default();
                //ensure it doesn't get here
                chai_1.assert.fail();
            }
            catch (err) {
                chai_1.assert.equal(err.name, "InvalidArgumentError");
                chai_1.assert.equal(err.message, "props");
            }
        });
        it("should throw InvalidArgumentError if props.id argument is not supplied", function () {
            try {
                // @ts-ignore
                var player = new player_1.default({});
                //ensure it doesn't get here
                chai_1.assert.fail();
            }
            catch (err) {
                chai_1.assert.equal(err.name, "InvalidArgumentError");
                chai_1.assert.equal(err.message, "props.id");
            }
        });
        it("should throw InvalidArgumentError if props.market argument is either not supplied, or is supplied but is not a function", function () {
            try {
                // @ts-ignore
                var player = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: emitter,
                });
                chai_1.assert.fail();
            }
            catch (err) {
                chai_1.assert.equal(err.name, "InvalidArgumentError");
                chai_1.assert.equal(err.message, "props.market");
            }
            try {
                // @ts-ignore
                var player = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: emitter,
                    // @ts-ignore
                    market: {},
                });
                chai_1.assert.fail();
            }
            catch (err) {
                chai_1.assert.equal(err.name, "InvalidArgumentError");
                chai_1.assert.equal(err.message, "props.market");
            }
        });
        it("should work when appropriate values are passed", function () {
            var player = new player_1.default({
                id: 1,
                // @ts-ignore
                validator: function (card) { return true; },
                emitter: emitter,
                market: mockMarket,
                pile: mockPile,
            });
        });
        it("should initialise emitter when not supplied in prop", function () {
            var props = {
                id: 1,
                // @ts-ignore
                validator: function (card) { return true; },
                market: mockMarket,
                pile: mockPile,
                emitter: undefined,
            };
            chai_1.assert.isUndefined(props.emitter);
            // @ts-ignore
            var player = new player_1.default(props);
            chai_1.assert.isDefined(props.emitter);
        });
    });
    describe("Cards", function () {
        var player = new player_1.default({
            id: 1,
            emitter: new events_1.EventEmitter(),
            market: mockMarket,
            pile: mockPile,
        });
        describe("Count", function () {
            it("should initially be 0", function () {
                chai_1.assert.equal(player.hand.length, 0);
            });
        });
        describe("Add", function () {
            it("should throw InvalidArgumentTypeError(Array) if invalid argument is supplied", function () {
                try {
                    // @ts-ignore
                    player.add({});
                    //ensure it doesn't get here
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "InvalidArgumentTypeError");
                    chai_1.assert.equal(err.type, Array);
                }
            });
            it("should add expected number of cards if supplied", function () {
                // @ts-ignore
                player.add([{}, {}, {}]);
                chai_1.assert.equal(player.hand().length, 3);
            });
        });
        describe("Move", function () {
            it("can match PickThree move", function () {
                var player = new player_1.default({
                    id: 1,
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: function () { return ({
                        // @ts-ignore
                        top: function () { return ({ move: moves_1.default.PickThree }); },
                    }); },
                });
                // @ts-ignore
                player.add([{ move: null }, { move: null }, { move: moves_1.default.PickThree }]);
                chai_1.assert.isTrue(player.canMatchMove());
            });
            it("can NOT match move", function () {
                var player = new player_1.default({
                    id: 1,
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: function () { return ({
                        // @ts-ignore
                        top: function () { return ({ move: moves_1.default.PickThree }); },
                    }); },
                });
                // @ts-ignore
                player.add([{ move: null }, { move: null }, { move: null }]);
                chai_1.assert.isFalse(player.canMatchMove());
            });
        });
        describe("Play", function () {
            it("should throw OutOfTurnError if not in turn", function () {
                try {
                    player.play(0);
                    //ensure it doesn't get here
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "OutOfTurnError");
                }
            });
            it("should throw CardNotFoundError if card index cannot be found", function () {
                try {
                    player.turn = true;
                    player.play(-1);
                    //ensure it doesn't get here
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "CardNotFoundError");
                }
            });
            it("should work", function () {
                var somePlayer = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: mockPile,
                });
                var card1 = (0, card_1.GetCircle)({ value: 4 });
                var card2 = (0, card_1.GetSquare)({ value: 2 });
                somePlayer.turn = true;
                somePlayer.add([card1, card2]);
                chai_1.assert.equal(somePlayer.hand().length, 2);
                somePlayer.play(0);
                chai_1.assert.equal(somePlayer.hand().length, 1);
            });
            it("should reach checkup", function () {
                var somePlayer = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: mockPile,
                });
                var card1 = (0, card_1.GetCircle)({ value: 4 });
                somePlayer.turn = true;
                somePlayer.add([card1]);
                chai_1.assert.equal(somePlayer.hand().length, 1);
                somePlayer.play(0);
                chai_1.assert.equal(somePlayer.hand().length, 0);
                chai_1.assert.isTrue(somePlayer.hasWon);
            });
            it("should pick from market and NOT reach checkup", function () {
                var somePlayer = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: mockPile,
                });
                var card1 = (0, card_1.GetCircle)({ value: 5 });
                somePlayer.turn = true;
                somePlayer.add([card1]);
                chai_1.assert.equal(somePlayer.hand().length, 1);
                somePlayer.play(0);
                chai_1.assert.equal(somePlayer.hand().length, 1);
                chai_1.assert.isFalse(somePlayer.hasWon);
            });
            it("should play iNeed", function () {
                var somePlayer = new player_1.default({
                    id: 1,
                    // @ts-ignore
                    validator: function (card) { return true; },
                    emitter: new events_1.EventEmitter(),
                    market: mockMarket,
                    pile: mockPile,
                });
                var card1 = (0, card_1.GetWhot)({
                    value: 20
                });
                var card2 = (0, card_1.GetCircle)({ value: 6 });
                somePlayer.turn = true;
                somePlayer.add([card1, card2]);
                somePlayer.play(0, shapes_1.default.Circle);
                chai_1.assert.equal(pile.top().shape, shapes_1.default.Whot);
                chai_1.assert.equal(pile.top().iNeed, shapes_1.default.Circle);
            });
            it("should throw PlayValidationFailedError", function () {
                try {
                    var somePlayer = new player_1.default({
                        id: 1,
                        // @ts-ignore
                        validator: function (card) { return true; },
                        emitter: new events_1.EventEmitter(),
                        market: mockMarket,
                        pile: mockPile,
                    });
                    var card1 = (0, card_1.GetSquare)({ value: 6 });
                    somePlayer.turn = true;
                    somePlayer.add([card1]);
                    somePlayer.play(0);
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "PlayValidationFailedError");
                }
            });
            it("should throw ExpectedToPickError", function () {
                try {
                    var somePlayer = new player_1.default({
                        id: 1,
                        // @ts-ignore
                        validator: function (card) { return true; },
                        emitter: new events_1.EventEmitter(),
                        market: mockMarket,
                        pile: mockPile,
                    });
                    var card1 = (0, card_1.GetCircle)({ value: 2 });
                    var card2 = (0, card_1.GetCircle)({ value: 6 });
                    somePlayer.turn = true;
                    somePlayer.add([card1, card2]);
                    somePlayer.play(0);
                    somePlayer.toPick = 2;
                    somePlayer.play(0);
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "ExpectedToPickError");
                }
            });
            it("should throw CardNeededUndefinedError", function () {
                try {
                    var somePlayer = new player_1.default({
                        id: 1,
                        // @ts-ignore
                        validator: function (card) { return true; },
                        emitter: new events_1.EventEmitter(),
                        market: mockMarket,
                        pile: mockPile,
                    });
                    var card1 = (0, card_1.GetWhot)({
                        value: 0
                    });
                    somePlayer.turn = true;
                    somePlayer.add([card1]);
                    somePlayer.play(0);
                    chai_1.assert.fail();
                }
                catch (err) {
                    chai_1.assert.equal(err.name, "CardNeededUndefinedError");
                }
            });
        });
    });
});
//# sourceMappingURL=player.test.js.map